/* 
 *   Genearate random error sequence
 *   Copyright (C) 1994, 1996 NTT Mobile Communications Network, Inc. (DoCoMo)
 *   All right reserved.
 *                      Toshio  Miki     (miki@mlab.nttdocomo.co.jp)
 *                      Toshiro Kawahara (kawahara@mlab.nttdocomo.co.jp)
 *  Name:  erand
 *  
 *  Usage: erand <iseed> <bitrate(b/s)> <ber> <input_file_name> <output_file_name>
 * 
 *  Example usage:
 *      - Generate random bit error
 *         Bit error rate: 1.0e-4
 *         Bitrate = 24k bits/sec
 *             % erand 10 24000 1.0e-4 input.seq error.seq
 *  Modify (Date: Jul. 25 1995) for  1.5 sec error free duration 
 *  Modify (Date: Oct. 10 1997) deleting error free duration for audio
 */
#include <stdio.h>
#define EFREESEC 0

float frand();

void
main(argc, argv)
     int argc;
     char **argv;
{
  FILE *fpin, *fpout;
  long iseed;
  float ber;
  int bitrate;
  int len;
  int nbits;
  unsigned char seq;
  int i,j;
  
  if(argc != 6){
    fprintf(stderr, "Usage: %s <iseed> <Bitrate> <BER> <Inputfile> <Outputfile>\n",argv[0]);
    fprintf(stderr, "\tFilename '-' indicates stdin/stdout.\n");
    fprintf(stderr, "\tWith input filename '+number',this program generate <number> bytes\n\terror sequence.\n");
    exit(-1);
  }
  
  sscanf(argv[1], "%ld", &iseed);
  sscanf(argv[2], "%d", &bitrate);
  sscanf(argv[3], "%f", &ber);
  
  if(argv[4][0] == '-')
    fpin = stdin;
  else if(argv[4][0] == '+'){
    fpin = NULL;
    sscanf(&argv[4][1], "%d", &len);
  }else{
    if(NULL == (fpin = fopen(argv[4],"rb"))){
      perror(argv[4]);
      exit(-1);
    }
  }

  if(argv[5][0] == '-')
    fpout = stdout;
  else{
    if(NULL == (fpout = fopen(argv[5],"wb"))){
      perror(argv[5]);
      exit(-1);
    }
  }

  nbits = bitrate * EFREESEC;	/* Number of error free bits */

  if(fpin != NULL){		/* Additional Error */
    while(fread(&seq, sizeof(seq), 1, fpin) == 1){
      for(i=0; i<8; i++){
	nbits --;
	if(nbits < 0)
	  seq ^= (frand(&iseed) < ber ? (0x80>>i) : 0);
      }
      fwrite(&seq, sizeof(seq), 1, fpout);
    }
  }else{
    for(i=0; i<len; i++){	/* Generate Error Pattern */
      seq = 0;
      for(j=0; j<8; j++){
	nbits --;
	if(nbits < 0)
	  seq |= (frand(&iseed) < ber ? (0x80>>j) : 0);
      }
      fwrite(&seq, sizeof(seq), 1, fpout);
    }
  }
  fprintf(stderr, "Iseed = %d\n", iseed);
}
  
      
