/*
 *   Generate random value
 *   Copyright (C) 1994 NTT Mobile Communications Network, Inc. (DoCoMo)
 *                      Toshio  Miki     (miki@mlab.nttdocomo.co.jp)
 *                      Toshiro Kawahara (kawahara@mlab.nttdocomo.co.jp)
 */
#include <stdio.h>

/*
 *  Generate (0.0, 1.0) random value
 *  Ref.  D.E.Knuth, "The Art of Computer Programming", Chap.3
 */
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float
frand(iseed)
     long *iseed;
{
  int i;
  long k;
  static int iy = 0;
  static long iv[NTAB];
  float tmp;

  if(iy == 0 || *iseed<= 0){	/* Initialize */
    if(*iseed == 0) *iseed = 141421356;	/* iseed must not 0 */
    for(i=NTAB+10; i>=0; i--){
      k = *iseed / IQ;
      *iseed = IA * (*iseed - k*IQ) - k*IR;
      if(*iseed < 0) *iseed += IM;
      if(i < NTAB) iv[i] = *iseed;
    }
    iy = iv[0];
  }
  k = *iseed/IQ;
  *iseed = IA * (*iseed - k*IQ) - k*IR;
  if(*iseed < 0) *iseed += IM;
  i = iy/NDIV;
  iy = iv[i];
  iv[i] = *iseed;
  tmp = AM*iy;
  if(tmp > RNMX) 
    return RNMX;
  else 
    return tmp;
}
